#import "XeeQuicktimeLoader.h"



@implementation XeeQuicktimeImage

-(id)initWithFile:(NSString *)name firstBlock:(NSData *)block attributes:(NSDictionary *)attributes view:(XeeView *)targetview;
{
	if(self=[super initWithFile:name firstBlock:block attributes:attributes view:targetview])
	{
		subimages=nil;

		CFURLRef url=CFURLCreateWithFileSystemPath(NULL,(CFStringRef)filename,kCFURLPOSIXPathStyle,FALSE);
		if(url)
		{
			FSRef fsref;
			if(CFURLGetFSRef(url,&fsref))
			{
				CFRelease(url);

				FSSpec fsspec;
				if(FSGetCatalogInfo(&fsref,kFSCatInfoNone,NULL,NULL,&fsspec,NULL)==noErr)
				{
					if(GetGraphicsImporterForFile(&fsspec,&gi)==noErr)
					{
						ImageDescriptionHandle desc;

						if(GraphicsImportGetImageDescription(gi,&desc)==noErr)
						{
							width=(*desc)->width;
							height=(*desc)->height;
							depth=(*desc)->depth;
							format=[[NSString alloc] initWithBytes:(*desc)->name+1 length:(*desc)->name[0] encoding:NSMacOSRomanStringEncoding];

							DisposeHandle((Handle)desc);

							if([format isEqual:@"Photo - JPEG"])
							{
								[format release];
								format=[[NSString alloc] initWithString:@"JPEG"]; // what the hell, Quicktime?
							}

							subimages=[[NSMutableArray alloc] initWithCapacity:16];
							nextindex=0;

							return self;
						}
					}
				}
			}
		}

		[self release];
	}

	return nil;
}

-(void)dealloc
{
	if(gi) CloseComponent(gi);

	[format release];
	[subimages release];

	[super dealloc];
}


static OSErr progress_func(short message,Fixed completeness,long refcon)
{
	XeeQuicktimeImage *image=(XeeQuicktimeImage *)refcon; // ow ow ow, 64-bit issues!

	if([image hasBeenStopped]) return codecAbortErr;
	return noErr;
}

-(BOOL)load
{
	EnterMoviesOnThread(0);

	CGrafPtr origport;
	GDHandle origdev;
	GetGWorld(&origport,&origdev);

	unsigned long count;
	GraphicsImportGetImageCount(gi,&count);

	struct ICMProgressProcRecord progrec={progress_func,(long)self}; // ouch, 64-bit issues!
	GraphicsImportSetProgressProc(gi,&progrec);

	GraphicsImportSetQuality(gi,codecLosslessQuality);
	while(nextindex<count)
	{
		if(stop) break;

		if(GraphicsImportSetImageIndex(gi,nextindex+1)==noErr)
		{
			ImageDescriptionHandle desc;
			if(GraphicsImportGetImageDescription(gi,&desc)==noErr)
			{
				int framewidth=(*desc)->width;
				int frameheight=(*desc)->height;
				DisposeHandle((Handle)desc);

				int type;
				if([format isEqual:@"JPEG"]) type=XeeImageYUV;
				else type=XeeImageRGB;

				XeeBitmapImage *image=[[[XeeBitmapImage alloc] initWithType:type width:framewidth height:frameheight view:view] autorelease];

				if(image)
				{
					short drawsall;

					if(GraphicsImportDoesDrawAllPixels(gi,&drawsall)!=noErr || drawsall!=graphicsImporterDrawsAllPixels)
					{
						[image setTransparent:YES];
						[image setBackgroundColor:[NSColor whiteColor]];
					}

					if(GraphicsImportSetGWorld(gi,[image gworld],NULL)==noErr)
					{
						ComponentResult res=GraphicsImportDraw(gi);
						if(res==noErr)
						{
							if([format isEqual:@"JPEG"]) [image fixYUVGamma]; 

							[image buildTextures];
							[subimages addObject:image];
						}
						else if(stop) break;
					}
				}
			}
		}
		nextindex++;
	}

	SetGWorld(origport,origdev);

	ExitMoviesOnThread();

	if(stop)
	{
		return NO;
	}

	CloseComponent(gi);
	gi=NULL;

	return YES;
}

-(BOOL)loaded
{
	return gi?NO:YES;
}



-(int)frames { return subimages?[subimages count]:0; }

-(void)setFrame:(int)frame
{
	if(!subimages||[subimages count]==0) return;

	if(frame<0) frame=0;
	if(frame>=[subimages count]) frame=[subimages count]-1;
	currindex=frame;
}

-(int)frame
{
	return currindex;
}

-(void)draw:(NSPoint)point size:(NSSize)size
{
	if(!subimages||[subimages count]==0) return;
	[[subimages objectAtIndex:currindex] draw:point size:size];
}

-(int)width
{
	if(subimages&&[subimages count]>0) return [(XeeBitmapImage *)[subimages objectAtIndex:currindex] width];
	else return width;
}

-(int)height
{
	if(subimages&&[subimages count]>0) return [(XeeBitmapImage *)[subimages objectAtIndex:currindex] height];
	else return height;
}

-(BOOL)transparent
{
	if(subimages&&[subimages count]>0) return [(XeeBitmapImage *)[subimages objectAtIndex:currindex] transparent];
	else return transparent;
}

-(NSColor *)backgroundColor
{
	if(subimages&&[subimages count]>0) return [(XeeBitmapImage *)[subimages objectAtIndex:currindex] backgroundColor];
	else return back;
}



+(NSArray *)fileTypes
{
	return [NSArray arrayWithObjects:
		@"jpg", // JPEG Image
		@"jpeg",
		@"jpe",
		@"'JPEG'",

		@"png", // Portable Network Graphics
		@"'PNG '", 
		@"'PNGf'", 

		@"gif", // Graphics Interchange Format
		@"'GIFf'",
		@"'GIF '",

		@"tif", // TIFF Image
		@"tiff", 
		@"'TIFF'", 

		@"bmp", // Windows Bitmap Image
		@"dib", //?
		@"'BMP '", 
		@"'BMPf'", 
		@"'BMPp'", 

		@"psd", // Adobe Photoshop Image
		@"'8BPS'",

		@"tga", // Targa Image
		@"'TPIC'", 

		@"jp2", // JPEG 2000 Image
		@"'jp2 '", 

		@"pict", // PICT Image
		@"pct", 
		@"pic",
		@"'PICT'", 

		@"fpx", // FlashPix Image
		@"'FPix'", 

		@"qtif", // Apple Quicktime Image
		@"qti",
		@"qif", 
		@"'qtif'", 

		@"sgi", // Silicon Graphics Image
		@"'SGI '", 
		@"'.SGI'", 

		@"pntg", // Apple MacPaint image
		@"'PNTG'", 

		@"mac", //?

		nil
	];
}

@end
